package ch.interlis.ilirepository.impl;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

import ch.ehi.basics.logging.EhiLogger;
import ch.interlis.ili2c.metamodel.TransferDescription;
import ch.interlis.ili2c.modelscan.IliFile;
import ch.interlis.ili2c.modelscan.IliModel;
import ch.interlis.iom_j.xtf.XtfModel;
import ch.interlis.iom_j.xtf.XtfWriterBase;
import ch.interlis.iox_j.EndBasketEvent;
import ch.interlis.iox_j.EndTransferEvent;
import ch.interlis.iox_j.ObjectEvent;
import ch.interlis.iox_j.StartBasketEvent;
import ch.interlis.iox_j.StartTransferEvent;
import ch.ehi.iox.ilisite.IliRepository09.RepositoryIndex.ModelMetadata;
import ch.ehi.iox.ilisite.IliRepository09.RepositoryIndex.ModelMetadata_SchemaLanguage;
import ch.ehi.iox.ilisite.IliRepository09.ModelName_;
import ch.ehi.iox.ilisite.ILIREPOSITORY09;

/** Tool to create an ilimodels.xml file in a given local folder tree.
 * @author ceis
 */
public class MakeIliModelsXml {
	/** name of application as shown to user.
	 */
	public static final String APP_NAME="mkilimodelsxml";
	/** name of jar file.
	 */
	public static final String APP_JAR="iox.jar"; 
	private static String version="1.0.0";

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			String outFile=null;
			int argi=0;
			for(;argi<args.length;argi++){
				String arg=args[argi];
				if(arg.equals("--trace")){
					EhiLogger.getInstance().setTraceFilter(false); 
				}else if (arg.equals("--quiet")){
					  ch.ehi.basics.logging.StdListener.getInstance().skipInfo(true);
					  continue;
				}else if(arg.equals("--version")){
					printVersion();
					return;
				}else if(arg.equals("--o")){
						argi++;
						outFile=args[argi];
						continue;
				}else if(arg.equals("--help")){
						printHelp();
						return;
					
				}else if(arg.startsWith("-")){
					EhiLogger.logAdaption(arg+": unknown option; ignored");
				}else{
					break;
				}
			}
			if(argi==args.length){
				printHelp();
				return;
			}
			if(argi+1!=args.length){
				printHelp();
				return;
			}
			String ilidir=args[argi];
			
			// scan folder
			HashSet models=scanIliFileDir(new File(ilidir));
			
			if(true){
				// write metadata
				if(outFile==null){
					outFile=ilidir+java.io.File.separator+"ilimodels.xml";
				}
				java.io.OutputStream outStream=null;
				XtfWriterBase ioxWriter=null;
				try{
					outStream=new java.io.FileOutputStream(outFile);
					ioxWriter = new XtfWriterBase( outStream,  ILIREPOSITORY09.getIoxMapping(),"2.3");
					ioxWriter.setModels(new XtfModel[]{ILIREPOSITORY09.getXtfModel()});
					StartTransferEvent startTransferEvent = new StartTransferEvent();
					startTransferEvent.setSender( APP_NAME+"-"+getVersion() );
					ioxWriter.write( startTransferEvent );
					StartBasketEvent startBasketEvent = new StartBasketEvent( ILIREPOSITORY09.RepositoryIndex, "b1");
					ioxWriter.write( startBasketEvent );
					Iterator modeli=models.iterator();
					while(modeli.hasNext()){
						ModelMetadata model=(ModelMetadata)modeli.next();
						ioxWriter.write(new ObjectEvent(model));
					}
					
					ioxWriter.write( new EndBasketEvent() );
					ioxWriter.write( new EndTransferEvent() );
					
					ioxWriter.flush();
				}catch(java.io.FileNotFoundException ex){
					EhiLogger.logError(ex);
					System.exit(1);
				}finally{
					if(ioxWriter!=null){
						ioxWriter.close();
						ioxWriter=null;
					}
					if(outStream!=null){
						try{
							outStream.close();				
						}catch(java.io.IOException ex){
							EhiLogger.logError(ex);
						}
						outStream=null;
					}
				}
			}
		} catch (Exception ex) {
			EhiLogger.logError(APP_NAME+": an internal error has occured",ex);
			System.exit(1);
		}
	}
	/** scans a directory for ili-files.
	 * @param startdir directory to scan
	 * @return set<IliFile>
	 */
	public static HashSet scanIliFileDir(File startdir)
	throws IOException
	{
		int oid=1;
		java.net.URI starturi=startdir.toURI();
		HashSet ret=new HashSet();
		ArrayList dirs=new ArrayList();
		dirs.add(startdir);
		ch.ehi.basics.view.GenericFileFilter filter=new ch.ehi.basics.view.GenericFileFilter("INTERLIS models (*.ili)","ili");
		while(!dirs.isEmpty()){
			File dir=(File)dirs.remove(0);
			File filev[]=dir.listFiles();
			for(int i=0;i<filev.length;i++){
				if(filev[i].isDirectory()){
					dirs.add(filev[i]);
					continue;
				}
				if(filter.accept(filev[i])){
					IliFile iliFile=ch.interlis.ili2c.ModelScan.scanIliFile(filev[i]);
					if(iliFile!=null){
						java.net.URI relPath=starturi.relativize(filev[i].toURI());
						String md5=RepositoryAccess.calcMD5(filev[i]);
						String fileVersion= new java.text.SimpleDateFormat("yyyy-MM-dd").format(new java.util.Date(filev[i].lastModified()));
						Iterator modeli=iliFile.iteratorModel();
						while(modeli.hasNext()){
							IliModel iliModel=(IliModel)modeli.next();
							ModelMetadata model=new ModelMetadata(Integer.toString(oid++));
							model.setFile(relPath.toString());
							model.setName(iliModel.getName());
							double cslVersion=iliModel.getIliVersion();
							if(cslVersion==1.0){
								model.setSchemaLanguage(ModelMetadata_SchemaLanguage.ili1);
							}else if(cslVersion==2.2){
								model.setSchemaLanguage(ModelMetadata_SchemaLanguage.ili2_2);
							}else if(cslVersion==2.3){
								model.setSchemaLanguage(ModelMetadata_SchemaLanguage.ili2_3);
							}else{
								throw new IllegalStateException("unexpected ili version");
							}
							model.setVersion(fileVersion);
							model.setmd5(md5);
							Iterator depi=iliModel.getDependencies().iterator();
							while(depi.hasNext()){
								String depModelName=(String)depi.next();
								ModelName_ depModel=new ModelName_();
								depModel.setvalue(depModelName);
								model.adddependsOnModel(depModel);
							}
							ret.add(model);
						}
					}
				}
			}
		}
		return ret;
	}
	private static void printHelp() {
		printVersion ();
		System.err.println();
		printDescription ();
		System.err.println();
		printUsage ();
		System.err.println();
		System.err.println("OPTIONS");
		System.err.println();
		//System.err.println("-gui                  start GUI.");
		System.err.println("--o filename           name of outputfile.");
		System.err.println("--trace                enable trace messages.");
  	    System.err.println("--quiet               Suppress info messages.");
		System.err.println("--help                 Display this help text.");
		System.err.println("--version              Display the version of pmtexp.");
		System.err.println();
	}
	protected static void printVersion ()
	{
	  System.err.println(APP_NAME+", Version "+getVersion());
	  System.err.println("  Developed by Eisenhut Informatik AG, CH-3401 Burgdorf");
	}


	protected static void printDescription ()
	{
	  System.err.println("DESCRIPTION");
	  System.err.println("  Reads INTERLIS models and generates a suitable ilimodels.xml file (according to IliRepository09.");
	}


	protected static void printUsage()
	{
	  System.err.println ("USAGE");
	  System.err.println("  java -jar "+APP_JAR+" [Options] folder-with-ili-files");
	}
	/** get version of program.
	 * @return version e.g. "1.0.0"
	 */
	public static String getVersion() {
		  if(version==null){
		java.util.ResourceBundle resVersion = java.util.ResourceBundle.getBundle("ch/ehi/ili2imd/Version");
			StringBuffer ret=new StringBuffer(20);
		ret.append(resVersion.getString("versionMajor"));
			ret.append('.');
		ret.append(resVersion.getString("versionMinor"));
			ret.append('.');
		ret.append(resVersion.getString("versionMicro"));
			ret.append('-');
		ret.append(resVersion.getString("versionDate"));
			version=ret.toString();
		  }
		  return version;
	}

}
